'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var zod = require('zod');
var pqb = require('pqb');
var pormSchemaToZod = require('porm-schema-to-zod');
var zodMock = require('@anatine/zod-mock');

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
const omit = (obj, keys) => {
  const res = __spreadValues({}, obj);
  Object.keys(keys).forEach((key) => {
    delete res[key];
  });
  return res;
};
const pick = (obj, keys) => {
  const res = {};
  Object.keys(keys).forEach((key) => {
    const value = obj[key];
    if (value !== void 0) {
      res[key] = value;
    }
  });
  return res;
};
const processCreateData = (factory, data, arg) => {
  const pick2 = {};
  for (const key in factory.model.shape) {
    pick2[key] = true;
  }
  factory.model.primaryKeys.forEach((key) => {
    if (factory.model.shape[key].dataType.includes("serial")) {
      delete pick2[key];
    }
  });
  const result = {};
  const fns = {};
  const allData = arg ? __spreadValues(__spreadValues({}, data), arg) : data;
  for (const key in allData) {
    delete pick2[key];
    const value = allData[key];
    if (typeof value === "function") {
      fns[key] = value;
    } else {
      result[key] = value;
    }
  }
  const pickedSchema = factory.schema.pick(pick2);
  return () => {
    Object.assign(result, zodMock.generateMock(pickedSchema));
    for (const key in fns) {
      result[key] = fns[key](factory.sequence);
    }
    factory.sequence++;
    return __spreadValues({}, result);
  };
};
const _TestFactory = class {
  constructor(model, schema, data = {}, options = {}) {
    this.model = model;
    this.schema = schema;
    this.data = data;
    this.omitValues = {};
    this.pickValues = {};
    var _a;
    if (options.sequence !== void 0) {
      this.sequence = options.sequence;
    } else {
      let workerId = parseInt(process.env.JEST_WORKER_ID);
      if (isNaN(workerId))
        workerId = 1;
      this.sequence = (workerId - 1) * ((_a = options.sequenceDistance) != null ? _a : 1e3) + 1;
    }
  }
  set(data) {
    return Object.assign(Object.create(this), {
      data: __spreadValues(__spreadValues({}, this.data), data)
    });
  }
  omit(keys) {
    return Object.assign(Object.create(this), {
      omitValues: __spreadValues(__spreadValues({}, this.omitValues), keys)
    });
  }
  pick(keys) {
    return Object.assign(Object.create(this), {
      pickValues: __spreadValues(__spreadValues({}, this.pickValues), keys)
    });
  }
  build(data) {
    let schema = this.schema;
    let arg = data ? __spreadValues(__spreadValues({}, this.data), data) : this.data;
    if (this.omitValues) {
      schema = schema.omit(this.omitValues);
      arg = omit(arg, this.omitValues);
    }
    if (this.pickValues && Object.keys(this.pickValues).length) {
      schema = schema.pick(this.pickValues);
      arg = pick(arg, this.pickValues);
    }
    const result = zodMock.generateMock(schema);
    for (const key in arg) {
      const value = arg[key];
      if (typeof value === "function") {
        result[key] = value(this.sequence);
      } else {
        result[key] = value;
      }
    }
    this.sequence++;
    return result;
  }
  buildList(qty, data) {
    return [...Array(qty)].map(() => this.build(data));
  }
  async create(data) {
    const getData = processCreateData(this, this.data, data);
    return await this.model.create(getData());
  }
  async createList(qty, data) {
    const getData = processCreateData(this, this.data, data);
    const arr = [...Array(qty)].map(getData);
    return await this.model.createMany(arr);
  }
  extend() {
    const { model, schema } = this;
    return class extends _TestFactory {
      constructor() {
        super(model, schema);
      }
    };
  }
};
let TestFactory = _TestFactory;
const nowString = new Date().toISOString();
const createFactory = (model, options) => {
  var _a, _b;
  const schema = pormSchemaToZod.instanceToZod(model);
  const data = {};
  const now = Date.now();
  for (const key in model.shape) {
    const column = model.shape[key];
    if (column instanceof pqb.DateBaseColumn) {
      if (column.data.as instanceof pqb.IntegerBaseColumn) {
        data[key] = (sequence) => now + sequence;
      } else if (((_a = column.parseFn) == null ? void 0 : _a.call(column, nowString)) instanceof Date) {
        data[key] = (sequence) => new Date(now + sequence);
      } else {
        data[key] = (sequence) => new Date(now + sequence).toISOString();
      }
    } else if (column instanceof pqb.TextBaseColumn) {
      const max = (_b = options == null ? void 0 : options.maxTextLength) != null ? _b : 1e3;
      const item = schema.shape[key];
      const string = item instanceof zod.ZodNullable ? item.unwrap() : item;
      const maxCheck = string._def.checks.find(
        (check) => check.kind === "max"
      );
      if (!maxCheck || maxCheck.value[0] > max) {
        schema.shape[key] = item instanceof zod.ZodNullable ? string.max(max).nullable() : string.max(max);
      }
    }
  }
  return new TestFactory(
    model,
    schema,
    data,
    options
  );
};

exports.TestFactory = TestFactory;
exports.createFactory = createFactory;
//# sourceMappingURL=index.js.map
